/*
 * @Author: kael 
 * @Date: 2020-11-05 18:19:03 
 * @Last Modified by: kael
 * @Last Modified time: 2020-11-11 10:58:45
 */

const Fetch = require('../fetch');
const Api = require('../api');

// https://github.com/Marak/colors.js/blob/56de9f0983f68cd0a08c5b76d10a783e4b881716/lib/styles.js
const codes = {
  red: [31, 39],
  gray: [90, 39],
  green: [32, 39],
  white: [37, 39],
};

function colors(string, name) {
  let [open, close] = codes[name];
  return `\u001b[${open}m${string}\u001b[${close}m`;
}

function moving_average(datas, days = 3) {
  if (!Array.isArray(datas)) return 0;
  let total = datas
    .slice(0, days)
    .map(v => parseFloat(v[2]))
    .reduce((sum, v) => sum += v, 0);
  return total / days;
}

// MA3 = MA5
// => (tm2 + tm1 + tm0) / 3 = (tm4 + tm3 + tm2 + tm1 + tm0) / 5
// => (tm2 + tm1 + tm0) * 5 = (tm4 + tm3 + tm2 + tm1 + tm0) * 3
// => 5tm2 + 5tm1 + 5tm0 - 3tm2 - 3tm1 - 3tm0 = 3tm4 + 3tm3
// => 2tm2 + 2tm1 + 2tm0 = 3tm4 + 3tm3
// => 2tm0 = 3tm4 + 3tm3 - 2tm2 - 2tm1
// => tm0 = 1.5tm4 + 1.5tm3 - tm2 - tm1
function marginal_value(datas) {
  let [tm4, tm3, tm2, tm1, tm0] = datas.slice(0, 5).map(v => parseFloat(v[2])).reverse();
  return {
    now: tm0,
    curr: 1.5 * tm4 + 1.5 * tm3 - tm2 - tm1,
    next: 1.5 * tm3 + 1.5 * tm2 - tm1 - tm0,
  };
}

const padding = [16, 16, 20, 20, 20, 20, 20];
const pad = (items) => {
  return items.map((item, index) => {
    return item[index ? 'padStart' : 'padEnd'](padding[index] - item.split(/[^\x00-\xff]/).length - 1);
  }).join('');
};

const LINE = new Array(120).fill('=').join('');

function start(stocks) {
  let codes = Array.from(new Set(stocks.map(v => v.code)));
  let quotes = codes.map(code => Api.quote.kline_dayqfq(code, 10));
  Promise.all(quotes).then(resps => {
    console.reset();
    console.log((new Date()).toTimeString().split(' ')[0]);
    console.log(LINE);
    console.log(pad(['ID', '名称', 'NOW', 'MA3', 'MA5', 'MVC', 'MVN']));
    console.log(LINE);
    let errors = resps.filter((data, index) => {
      let { id, info, data: resp } = data;
      let name = String(info[1] || '').replace(/(ETF|\s).+$/i, '');
      let MA3 = moving_average(resp, 3);
      let MA5 = moving_average(resp, 5);
      let MV = marginal_value(resp);
      if (!MV.now) return global.MARKET[id] = `us${info[2]}`;
      let row = pad([id, name, MV.now.toFixed(3), MA3.toFixed(3), MA5.toFixed(3), MV.curr.toFixed(3), MV.next.toFixed(3)]);
      console.log(colors(row, MA3 > MA5 ? 'red' : MA3 < MA5 ? 'green' : 'white'));
    });
    console.log(LINE);
    if (errors.length) throw new Error('Errors occur.');
    setTimeout(() => start(stocks), 10 * 1000);
  }).catch(() => {
    setTimeout(() => start(stocks), 1 * 1000);
  });
}

module.exports = () => {
  Fetch.futu().then(([holders, follows]) => {
    let stocks = [
      // ...Object.values(holders),
      ...Object.values(follows),
    ].flat();
    start(stocks);
  });
};